@@ -7,7 +7,7 @@ use warnings;
 use Scalar::Util 'blessed';
 use Carp         'confess';
 
-our $VERSION   = '0.64_07';
+our $VERSION   = '0.69_01';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -27,15 +27,15 @@ sub initialize {
     # we hand-construct the class 
     # until we can bootstrap it
     if ( my $meta = Class::MOP::get_metaclass_by_name($package_name) ) {
-       return $meta;
+        return $meta;
     } else {
-       my $meta = ( ref $class || $class )->_new({
-           'package'   => $package_name,
-       });
+        my $meta = ( ref $class || $class )->_new({
+            'package'   => $package_name,
+        });
 
-       Class::MOP::store_metaclass_by_name($package_name, $meta);
+        Class::MOP::store_metaclass_by_name($package_name, $meta);
 
-       return $meta;
+        return $meta;
     }
 }
 
@@ -276,6 +276,10 @@ sub list_all_package_symbols {
 
 sub get_all_package_symbols {
     my ($self, $type_filter) = @_;
+
+    die "Cannot call get_all_package_symbols as a class method"
+        unless ref $self;
+
     my $namespace = $self->namespace;
 
     return %$namespace unless defined $type_filter;
